# Copyright 2010, 2011 Kevin Ryde

# This file is part of Math-Image.
#
# Math-Image is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# Math-Image is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Math-Image.  If not, see <http://www.gnu.org/licenses/>.

package App::MathImage::NumSeq::Sequence::CountPrimeFactors;
use 5.004;
use strict;
use List::Util 'min', 'max';

use App::MathImage::NumSeq::Base '__';
use base 'App::MathImage::NumSeq::Sequence';

use vars '$VERSION';
$VERSION = 57;

# uncomment this to run the ### lines
#use Smart::Comments;

use constant name => __('Count Prime Factors');
use constant description => __('Count of prime factors.');
use constant type_hash => { count => 1 };
use constant values_min => 1;

use constant parameter_list => ({ name    => 'multiplicity',
                                  display => __('Multiplicity'),
                                  type    => 'enum',
                                  choices => ['repeated','distinct'],
                                  default => 'repeated',
                                  # description => __(''),
                                },
                               );

# OeisCatalogue: A001221 multiplicity=0
# OeisCatalogue: A001222 multiplicity=1
sub oeis_anum {
  my ($self) = @_;
  return ($self->{'multiplicity'}
          ? 'A001222'   # with multiplicity
          : 'A001221'); # without multiplicity
}


sub rewind {
  my ($self) = @_;
  my $hi = $self->{'hi'};
  $self->{'i'} = 1;
  $self->{'string'} = "\0" x ($self->{'hi'}+1);
  vec ($self->{'string'}, 1,4) = 1;   # N=1 count 1
  while ($self->{'i'} < $self->{'lo'}-1) {
    $self->next;
  }
  return $self;
}

sub next {
  my ($self) = @_;

  my $i = $self->{'i'}++;
  my $hi = $self->{'hi'};
  if ($i > $hi) {
    return;
  }
  my $cref = \$self->{'string'};

  my $ret = vec ($$cref, $i,4);
  if ($ret == 0) {
    $ret++;
    # a prime
    for (my $power = 1; ; $power++) {
      my $step = $i ** $power;
      last if ($step > $hi);
      for (my $j = $step; $j <= $hi; $j += $step) {
        vec($$cref, $j,4) = min (15, vec($$cref,$j,4)+1);
      }
      last if $self->{'multiplicity'} eq 'distinct';
    }
    # print "applied: $i\n";
    # for (my $j = 0; $j < $hi; $j++) {
    #   printf "  %2d %2d\n", $j, vec($$cref, $j,4));
    # }
  }
  return ($i, $ret);
}

sub ith {
  my ($self, $i) = @_;
  ### CountPrimeFactors pred(): $i
  if ($self->{'i'} <= $i) {
    ### extend from: $self->{'i'}
    my $upto;
    while ((($upto) = $self->next)
           && $upto < $i) { }
  }
  return vec($self->{'string'}, $i,4);
}

sub pred {
  my ($self, $value) = @_;
  return ($value >= 0);
}

1;
__END__



# Untouchables, not sum of proper divisors of any other integer
# p*q sum S=1+p+q
# so sums up to hi need factorize to (hi^2)/4
# 
